@@ -11,9 +11,8 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
-
-from data_warehouse_client import table_writer_json
+from data_warehouse_client.table_writer_json import data_warehouse_metadata_tables_to_dictionary, \
+    write_tables_in_dw_from_dictionary
 
 
 def clone_study_metadata(dw, source_study_id, dest_study_id):
@@ -25,8 +24,8 @@ def clone_study_metadata(dw, source_study_id, dest_study_id):
     :return total rows inserted
     """
     # extract existing metadata
-    tbls = table_writer_json.data_warehouse_metadata_tables_to_dictionary(dw, source_study_id)
+    tbls = data_warehouse_metadata_tables_to_dictionary(dw, source_study_id)
     for table in tbls:  # for each table
         for row in tbls[table]:  # for each row
             row.update({'study': dest_study_id})  # update all occurrences of study to the destination study
-    return table_writer_json.write_tables_in_dw_from_dictionary(dw, tbls)  # write the result to the warehouse
+    return write_tables_in_dw_from_dictionary(dw, tbls)  # write the result to the warehouse
