@@ -22,11 +22,13 @@
 # no ordinal, nominal, bounded integer or bounded real values are out of bounds
 
 
-from data_warehouse_client import file_utils
-from data_warehouse_client import data_warehouse
-from data_warehouse_client import print_io
 from tabulate import tabulate
-import datetime
+from datetime import datetime
+
+from data_warehouse_client.check_for_datetime_table import datetimebounds_table_in_dw
+from data_warehouse_client.data_warehouse import core_sql_for_measurements
+from data_warehouse_client.file_utils import process_sql_template
+from data_warehouse_client.print_io import print_measurements, print_measurements_to_file
 
 
 def check_category_exists(dw, study):
@@ -37,7 +39,7 @@ def check_category_exists(dw, study):
     :return: the ids and names of measurement types in the study that fail the test
     """
     mappings = {"study": str(study)}
-    query = file_utils.process_sql_template("ordinal_types_not_matching_category.sql", mappings)
+    query = process_sql_template("ordinal_types_not_matching_category.sql", mappings)
     return dw.return_query_result(query)
 
 
@@ -49,7 +51,7 @@ def check_integer_bounds_exist(dw, study):
     :return: the ids and names of measurement types in the study that fail the test
     """
     mappings = {"study": str(study)}
-    query = file_utils.process_sql_template("bounded_integer_measurement_types_without_bounds.sql", mappings)
+    query = process_sql_template("bounded_integer_measurement_types_without_bounds.sql", mappings)
     return dw.return_query_result(query)
 
 
@@ -61,7 +63,19 @@ def check_real_bounds_exist(dw, study):
     :return: the ids and names of measurement types in the study that fail the test
     """
     mappings = {"study": str(study)}
-    query = file_utils.process_sql_template("bounded_real_measurement_types_without_bounds.sql", mappings)
+    query = process_sql_template("bounded_real_measurement_types_without_bounds.sql", mappings)
+    return dw.return_query_result(query)
+
+
+def check_datetime_bounds_exist(dw, study):
+    """
+    Find measurement types of bounded datetime value type without entry in boundsreal table
+    :param dw: handle to data warehouse
+    :param study: study id
+    :return: the ids and names of measurement types in the study that fail the test
+    """
+    mappings = {"study": str(study)}
+    query = process_sql_template("bounded_datetime_measurement_types_without_bounds.sql", mappings)
     return dw.return_query_result(query)
 
 
@@ -72,8 +86,8 @@ def check_valtype_matches_values(dw, study):
     :param study: study id
     :return: the measurements in the study that fail the test
     """
-    mappings = {"study": str(study), "core_sql": data_warehouse.core_sql_for_measurements()}
-    query = file_utils.process_sql_template("measurements_lacking_value.sql", mappings)
+    mappings = {"study": str(study), "core_sql": core_sql_for_measurements()}
+    query = process_sql_template("measurements_lacking_value.sql", mappings)
     return dw.return_query_result(query)
 
 
@@ -84,8 +98,8 @@ def check_category_in_range(dw, study):
     :param study: study id
     :return: the ids of measurements in the study that fail the test
     """
-    mappings = {"study": str(study), "core_sql": data_warehouse.core_sql_for_measurements()}
-    query = file_utils.process_sql_template("measurements_lacking_value.sql", mappings)
+    mappings = {"study": str(study), "core_sql": core_sql_for_measurements()}
+    query = process_sql_template("measurements_lacking_value.sql", mappings)
     return dw.return_query_result(query)
 
 
@@ -97,7 +111,7 @@ def check_bounded_integers(dw, study):
     :return: the ids of measurements in the study that fail the test
     """
     mappings = {"study": str(study)}
-    query = file_utils.process_sql_template("bounded_integers.sql", mappings)
+    query = process_sql_template("bounded_integers.sql", mappings)
     return dw.return_query_result(query)
 
 
@@ -109,7 +123,19 @@ def check_bounded_reals(dw, study):
     :return: the ids of measurements in the study that fail the test
     """
     mappings = {"study": str(study)}
-    query = file_utils.process_sql_template("bounded_reals.sql", mappings)
+    query = process_sql_template("bounded_reals.sql", mappings)
+    return dw.return_query_result(query)
+
+
+def check_bounded_datetimes(dw, study):
+    """
+    Returns the ids of measurements that hold bounded reals that are out of range
+    :param dw: handle to data warehouse
+    :param study: study id
+    :return: the ids of measurements in the study that fail the test
+    """
+    mappings = {"study": str(study)}
+    query = process_sql_template("bounded_datetimes.sql", mappings)
     return dw.return_query_result(query)
 
 
@@ -154,7 +180,7 @@ def print_check_warehouse(dw, study):
     r1 = check_valtype_matches_values(dw, study)
     n_invalid_entries = len(r1)
     if n_invalid_entries > 0:
-        print_io.print_measurements(r1)
+        print_measurements(r1)
     print(f'({n_invalid_entries} invalid entries)')
 
     print(f'-- Measurements declared as ordinal or nominal that refer to a non-existent category')
@@ -179,7 +205,16 @@ def print_check_warehouse(dw, study):
     if n_errors > 0:
         print(tabulate(r5, headers=['Id', 'Value', 'MeasurementType', 'Group', 'Min', 'Max', 'Participant']))
     print(f'({n_errors} measurements)')
+
     print()
+    if datetimebounds_table_in_dw(dw):
+        print(f'-- Measurements declared as Bounded Datetimes whose value is outside of the bounds')
+        r5 = check_bounded_datetimes(dw, study)
+        n_errors = len(r5)
+        if n_errors > 0:
+            print(tabulate(r5, headers=['Id', 'Value', 'MeasurementType', 'Group', 'Min', 'Max', 'Participant']))
+        print(f'({n_errors} measurements)')
+        print()
 
 
 def mk_txt_report_file_name(f_dir, report_name, time_string):
@@ -195,7 +230,7 @@ def print_check_warehouse_to_file(dw, study):
     """
 
     file_dir = "reports/"
-    timestamp = datetime.datetime.now()  # use the current date and time if none is specified
+    timestamp = datetime.now()  # use the current date and time if none is specified
     time_fname_str = timestamp.strftime('%Y-%m-%dh%Hm%Ms%S')
     fname = mk_txt_report_file_name(file_dir, "warehouse-check-", time_fname_str)
 
@@ -231,7 +266,7 @@ def print_check_warehouse_to_file(dw, study):
         r1 = check_valtype_matches_values(dw, study)
         n_invalid_entries = len(r1)
         if n_invalid_entries > 0:
-            print_io.print_measurements_to_file(r1, f)
+            print_measurements_to_file(r1, f)
         print(f'({n_invalid_entries} invalid entries)\n', file=f)
 
         print(f'-- Measurements declared as ordinal or nominal that refer to a non-existent category', file=f)
@@ -245,7 +280,8 @@ def print_check_warehouse_to_file(dw, study):
         r4 = check_bounded_integers(dw, study)
         n_errors = len(r4)
         if n_errors > 0:
-            print(tabulate(r4, headers=['Id', 'Value']), file=f)
+            print(tabulate(r4, headers=['Id', 'Value', 'MeasurementType', 'Group', 'Min', 'Max', 'Participant']),
+                  file=f)
         print(f'({n_errors} measurements)\n', file=f)
 
         print(f'-- Measurements declared as Bounded Reals whose value is outside of the bounds', file=f)
@@ -255,3 +291,12 @@ def print_check_warehouse_to_file(dw, study):
             print(tabulate(r5, headers=['Id', 'Value', 'MeasurementType', 'Group', 'Min', 'Max', 'Participant']),
                   file=f)
         print(f'({n_errors} measurements)\n', file=f)
+
+        if datetimebounds_table_in_dw(dw):
+            print(f'-- Measurements declared as Bounded Datetimes whose value is outside of the bounds', file=f)
+            r5 = check_bounded_datetimes(dw, study)
+            n_errors = len(r5)
+            if n_errors > 0:
+                print(tabulate(r5, headers=['Id', 'Value', 'MeasurementType', 'Group', 'Min', 'Max', 'Participant']),
+                      file=f)
+            print(f'({n_errors} measurements)\n', file=f)
